import aiohttp
import urllib.parse
import json
from typing import Any


class ApiResponse:
    def __init__(self, status_code, body):
        self.status = status_code
        self.body = body

    def __str__(self):
        return json.dumps({"status": self.status, "body": self.body})


async def send_request(url: str, request_type: str, headers: dict, query: dict = None, post_data: Any = '') -> ApiResponse:
    if query:
        query_string = urllib.parse.urlencode(query)
        url += '?' + query_string

    response_body = ''
    async with aiohttp.ClientSession() as session:
        if request_type == "GET":
            async with session.get(url, headers=headers) as response:
                response_body = await response.text()
                response_code = response.status
        elif request_type in ["POST", "PUT"]:  # Handle both POST and PUT
            post_data_dict = get_post_data_dict(post_data)
            post_data_json = json.dumps(post_data_dict)
            if request_type == "POST":
                async with session.post(url, data=post_data_json, headers=headers) as response:
                    response_body = await response.text()
                    response_code = response.status
            if request_type == "PUT":
                async with session.put(url, data=post_data_json, headers=headers) as response:
                    response_body = await response.text()
                    response_code = response.status

    try:
        response_body_json = json.loads(response_body)
    except json.JSONDecodeError:
        response_body_json = None

    return ApiResponse(status_code=response_code, body=response_body_json)


def get_post_data_dict(post_data: Any) -> dict:
    if type(post_data) == dict:
        return post_data
    return post_data.__dict__



import subprocess
import asyncio

START_VIRUAL_MACHINE = "common/shell_scripts/start-virtual-machine.sh"
START_ADB_APP = "common/shell_scripts/start-adb-app.sh"
START_NPM_AT_PATH = "common/shell_scripts/start-npm-at-path.sh"
START_FIREFOX_AT_URL = "common/shell_scripts/start-firefox-at-url.sh"
KILL_PROCESS_AT_PORT = "common/shell_scripts/kill-process-at-port.sh"


def start_virtual_machine(virtual_device_id: str) -> bool:
    process = subprocess.run(
        [START_VIRUAL_MACHINE, virtual_device_id], check=True)
    return proccess_end_handling(process.returncode)


def start_adb_app(package_name: str):
    process = subprocess.run([START_ADB_APP, package_name])
    return proccess_end_handling(process.returncode)


async def start_npm_at_path(path: str, search_message: str = ''):
    process = await asyncio.create_subprocess_exec(
        START_NPM_AT_PATH, path,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=False
    )

    try:
        while True:
            output_bytes = await process.stdout.readline()
            output = output_bytes.decode('utf-8')
            if output == '' and process.poll() is not None:
                break
            if search_message and search_message in output:
                break
            if output:
                print(output.strip())
            await asyncio.sleep(1)
    except Exception as e:
        print(f"Error in waiting for process to end {e}", flush=True)
        return proccess_end_handling(1)
    finally:
       print("Process ended")
       return proccess_end_handling(0)


def start_firefox_at_url(url: str):
    process = subprocess.run([START_FIREFOX_AT_URL, url])
    return proccess_end_handling(process.returncode)


def kill_process_at_port(port: int):
    process = subprocess.run(
        [KILL_PROCESS_AT_PORT, str(port)])
    return proccess_end_handling(process.returncode)


def kill_proccess(name: str):
    process = subprocess.run(["pkill", name])
    return proccess_end_handling(process.returncode)


def kill_adb_app(package_name: str):
    process = subprocess.run(
        ["adb", "shell", "am", "force-stop", package_name])
    return proccess_end_handling(process.returncode)


def proccess_end_handling(return_code: str) -> bool:
    success = False
    if return_code == 0:
        print("Script completed successfully")
        success = True
    else:
        print("Script failed with exit code", return_code)
    return success



from . import file_utils
from . import http_wrapper
from . import image_downloader
from . import shell_scripts_handler
from . import task_completion_source
from . import wake_machine
from . import web_socket_listener

__version__ = "0.1.0"

__all__ = [
    "file_utils",
    "http_wrapper",
    "image_downloader",
    "shell_scripts_handler",
    "task_completion_source",
    "wake_machine",
    "web_socket_listener",
]


import os

def list_files_in_directory(base_path):
    files = []
    for root, dirs, filenames in os.walk(base_path):
        for filename in filenames:
            # Get the full path
            full_path = os.path.join(root, filename)
            # Get the relative path from the config directory
            rel_path = os.path.relpath(full_path, base_path)
            files.append(rel_path)
    return files


import asyncio

class TaskCompletionSource:
    def __init__(self):
        self._future = None

    def set_result(self, result):
        if not self._future.done():
            self._future.set_result(result)
        else:
            raise RuntimeError("Result already set")

    def set_exception(self, exception):
        if not self._future.done():
            self._future.set_exception(exception)
        else:
            raise RuntimeError("Exception already set")

    def is_done(self) -> bool:
        return self._future.done()

    async def get_future(self):
        if self._future is None:
            self._future = asyncio.get_event_loop().create_future()
        return await self._future



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

try:
    with open("requirements.txt", "r", encoding="utf-8") as fh:
        requirements = [line.strip() for line in fh if line.strip() and not line.startswith("#")]
except FileNotFoundError:
    requirements = []

setup(
    name="thetemani-common-utils",
    version="0.1.3",
    author="Omer Hadad",
    description="A collection of common utility functions and scripts",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(include=['common_utils', 'common_utils.*']),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    license="MIT",
    project_urls={
        "Homepage": "https://github.com/Omer-hadad-s-Projects/common",
        "Bug Reports": "https://github.com/Omer-hadad-s-Projects/common/issues",
        "Source": "https://github.com/Omer-hadad-s-Projects/common",
    },
    python_requires=">=3.6",
    install_requires=requirements,
    include_package_data=True,
    package_data={
        "common_utils": ["shell_scripts/*.sh"],
    },
)


import websockets
import json
import os
import signal
from typing import Callable
import asyncio


class WebSocketServer:
    def __init__(self, port: int):
        self.port = port
        self.stop_event = asyncio.Event()
        self.on_message_received = None

    def change_message_received_callback(self, on_message_received: Callable):
        self.on_message_received = on_message_received

    def clear_on_message_received(self):
        self.on_message_received = None

    async def start(self):
        await self._start_websocket_listener()

    async def _start_websocket_listener(self):
        async def handler(websocket):
            while True:
                try:
                    data = await websocket.recv()
                    data_obj = json.loads(data)
                    if data_obj.get("Close"):
                        print(f"Closing web socket at port {self.port}")
                        await self.send_close_request()
                        break
                    else:
                        self.on_message_received(data)
                except websockets.ConnectionClosed:
                    print("Connection closed by the server")
                    break

        server = await websockets.serve(handler, "localhost", self.port)
        print(f"WebSocket server started on port {self.port}")
        await self.stop_event.wait()
        server.close()
        await server.wait_closed()
        print(f"WebSocket server stopped on port {self.port}")
        self.free_port()

    async def send_close_request(self):
        uri = f"ws://localhost:{self.port}"
        async with websockets.connect(uri) as websocket:
            await websocket.send(json.dumps({"Close": True}))
            print(f"Sent WebSocket close request to server on port {
                  self.port}")
            await websocket.close()

    def free_port(self):
        while True:
            pids = self.get_pids_using_port(self.port)
            if pids:
                for pid in pids:
                    print(f"Killing process with PID {pid} to free the port.")
                    try:
                        os.kill(pid, signal.SIGTERM)
                    except Exception as e:
                        print(f"Failed to kill process {pid}: {e}")
            else:
                print(f"No processes found using port {self.port}.")
                break

    @staticmethod
    def get_pids_using_port(port):
        try:
            result = os.popen(f"lsof -nP -t -i:{port}").read().strip()
            if result:
                return [int(pid) for pid in result.splitlines()]
            else:
                return []
        except Exception as e:
            print(f"Error getting PIDs for port {port}: {e}")
            return []



from common import http_wrapper


async def wake_machine(mac_address, ip_address, wake_on_lan_api_url) -> bool:
    post_data = {
        "mac_address": mac_address,
        "ip_address": ip_address
    }
    headers = {
        "Content-Type": "application/json",
        "Accept": "/",
        "Connection": "keep-alive",
    }
    request_url = wake_on_lan_api_url + "/wake"
    response = await http_wrapper.send_request(request_url, "POST", headers, None, post_data)
    if (response.status != 200):
        print(f"Failed to wake machine: {response.status} trying again")
        response = await http_wrapper.send_request(request_url, "POST", headers, None, post_data)
        print(f"Second attempt response: {response}")

    return response.status == 200 and response.body["exit_code"] == 0



import aiohttp
import os


async def download_image(url: str, save_path: str):
    try:
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as response:
                response.raise_for_status()
                with open(save_path, 'wb') as file:
                    while True:
                        chunk = await response.content.read(8192)
                        if not chunk:
                            break
                        file.write(chunk)

    except aiohttp.ClientError as e:
        print(f"An error occurred: {e}")


async def download_images(urls: list[str], images_directory: str):
    for url in urls:
        filename = url.split('/')[-1]
        save_path = os.path.join(images_directory, filename)
        await download_image(url, save_path)
